@@ -69,6 +69,11 @@ if (program.transport === "http") {
   type = "http";
 }
 
+if (program.transport === "websocket") {
+  program.transport = "buffered";
+  type = "websocket";
+}
+
 const options = {
   transport: helpers.transports[program.transport],
   protocol: helpers.protocols[program.protocol]
